//
// Created by huli on 2020/7/2.
//

#include "system_executor.h"


System_executor::System_executor()
{

}

System_executor::~System_executor()
{
	system_executor_uninit();
}

//初始化
Error_manager System_executor::system_executor_init(int threads_size)
{
	m_thread_pool.thread_pool_init(threads_size);
	m_system_executor_status = SYSTEM_EXECUTOR_READY;
	return Error_code::SUCCESS;
}

//反初始化
Error_manager System_executor::system_executor_uninit()
{
	m_thread_pool.thread_pool_uninit();
	m_system_executor_status = SYSTEM_EXECUTOR_UNKNOW;
	return Error_code::SUCCESS;
}

//检查消息是否有效, 主要检查消息类型和接受者, 判断这条消息是不是给我的.
Error_manager System_executor::check_msg(Communication_message* p_msg)
{
	if ( p_msg == NULL )
	{
		return Error_manager(Error_code::POINTER_IS_NULL, Error_level::MINOR_ERROR,
							 "  POINTER IS NULL ");
	}
	

		//检查消息类型
		switch ( p_msg->get_message_type() )
		{
			case Communication_message::Message_type::eDispatch_request_msg:
			{
				message::Dispatch_request_msg t_dispatch_request_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_dispatch_request_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_dispatch_request_msg =  "<< t_dispatch_request_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eDispatch_response_msg:
			{
				message::Dispatch_response_msg t_dispatch_response_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_dispatch_response_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_dispatch_response_msg =  "<< t_dispatch_response_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eDispatch_status_msg:
			{
				message::Dispatch_terminal_status_msg t_dispatch_terminal_status_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_dispatch_terminal_status_msg.ParseFromString(p_msg->get_message_buf()))
				{
					if ( t_dispatch_terminal_status_msg.id_struct().unit_id() == 0 )
					{
						LOG(INFO) << " t_dispatch_terminal_status_msg =  "<< t_dispatch_terminal_status_msg.DebugString() << " --  " << this;
					}
				}
				break;
			}

//			case Communication_message::Message_type::eParkspace_allocation_status_msg:
//			{
//				message::Parkspace_allocation_status_msg t_Parkspace_allocation_status_msg;
//				//针对消息类型, 对消息进行二次解析
//				if (t_Parkspace_allocation_status_msg.ParseFromString(p_msg->get_message_buf()))
//				{
//					LOG(INFO) << " t_Parkspace_allocation_status_msg =  "<< t_Parkspace_allocation_status_msg.DebugString() << " --  " << this;
//				}
//				break;
//			}
			case Communication_message::Message_type::eParkspace_allocation_request_msg:
			{
				message::Parkspace_allocation_request_msg t_Parkspace_allocation_request_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Parkspace_allocation_request_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Parkspace_allocation_request_msg =  "<< t_Parkspace_allocation_request_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eParkspace_allocation_response_msg:
			{
				message::Parkspace_allocation_response_msg t_Parkspace_allocation_response_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Parkspace_allocation_response_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Parkspace_allocation_response_msg =  "<< t_Parkspace_allocation_response_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eParkspace_search_request_msg:
			{
				message::Parkspace_search_request_msg t_Parkspace_search_request_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Parkspace_search_request_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Parkspace_search_request_msg =  "<< t_Parkspace_search_request_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eParkspace_search_response_msg:
			{
				message::Parkspace_search_response_msg t_Parkspace_search_response_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Parkspace_search_response_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Parkspace_search_response_msg =  "<< t_Parkspace_search_response_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eParkspace_release_request_msg:
			{
				message::Parkspace_release_request_msg t_Parkspace_release_request_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Parkspace_release_request_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Parkspace_release_request_msg =  "<< t_Parkspace_release_request_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eParkspace_release_response_msg:
			{
				message::Parkspace_release_response_msg t_Parkspace_release_response_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Parkspace_release_response_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Parkspace_release_response_msg =  "<< t_Parkspace_release_response_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eParkspace_force_update_request_msg:
			{
				message::Parkspace_force_update_request_msg t_Parkspace_force_update_request_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Parkspace_force_update_request_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Parkspace_force_update_request_msg =  "<< t_Parkspace_force_update_request_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eParkspace_force_update_response_msg:
			{
				message::Parkspace_force_update_response_msg t_Parkspace_force_update_response_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Parkspace_force_update_response_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Parkspace_force_update_response_msg =  "<< t_Parkspace_force_update_response_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eParkspace_confirm_alloc_request_msg:
			{
				message::Parkspace_confirm_alloc_request_msg t_Parkspace_confirm_alloc_request_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Parkspace_confirm_alloc_request_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Parkspace_confirm_alloc_request_msg =  "<< t_Parkspace_confirm_alloc_request_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eParkspace_confirm_alloc_response_msg:
			{
				message::Parkspace_confirm_alloc_response_msg t_Parkspace_confirm_alloc_response_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Parkspace_confirm_alloc_response_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Parkspace_confirm_alloc_response_msg =  "<< t_Parkspace_confirm_alloc_response_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eParkspace_refresh_request_msg:
			{
				message::Parkspace_refresh_request_msg t_Parkspace_refresh_request_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Parkspace_refresh_request_msg.ParseFromString(p_msg->get_message_buf()))
				{
//					LOG(INFO) << " t_Parkspace_refresh_request_msg =  "<< t_Parkspace_refresh_request_msg.DebugString() << " --  " << this;
					LOG(INFO) << " t_Parkspace_refresh_request_msg =  "<< t_Parkspace_refresh_request_msg.base_info().msg_type() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eParkspace_allocation_data_response_msg:
			{
				message::Parkspace_allocation_data_msg t_Parkspace_allocation_data_response_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Parkspace_allocation_data_response_msg.ParseFromString(p_msg->get_message_buf()))
				{
//					LOG(INFO) << " t_Parkspace_allocation_data_response_msg =  "<< t_Parkspace_allocation_data_response_msg.DebugString() << " --  " << this;
//					LOG(INFO) << " t_Parkspace_allocation_data_response_msg =  "<< t_Parkspace_allocation_data_response_msg.base_info().msg_type() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eParkspace_manual_search_request_msg:
			{
				message::Parkspace_manual_search_request_msg t_Parkspace_manual_search_request_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Parkspace_manual_search_request_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Parkspace_manual_search_request_msg =  "<< t_Parkspace_manual_search_request_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eParkspace_manual_search_response_msg:
			{
				message::Parkspace_manual_search_response_msg t_Parkspace_manual_search_response_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Parkspace_manual_search_response_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Parkspace_manual_search_response_msg =  "<< t_Parkspace_manual_search_response_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eStore_command_request_msg:
			{
				message::Store_command_request_msg t_Store_command_request_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Store_command_request_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Store_command_request_msg =  "<< t_Store_command_request_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eStore_command_response_msg:
			{
				message::Store_command_response_msg t_Store_command_response_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Store_command_response_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Store_command_response_msg =  "<< t_Store_command_response_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::ePickup_command_request_msg:
			{
				message::Pickup_command_request_msg t_Pickup_command_request_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Pickup_command_request_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Pickup_command_request_msg =  "<< t_Pickup_command_request_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::ePickup_command_response_msg:
			{
				message::Pickup_command_response_msg t_Pickup_command_response_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Pickup_command_response_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Pickup_command_response_msg =  "<< t_Pickup_command_response_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eEntrance_manual_operation_msg:
			{
				message::Entrance_manual_operation_msg t_Entrance_manual_operation_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Entrance_manual_operation_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Entrance_manual_operation_msg =  "<< t_Entrance_manual_operation_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eProcess_manual_operation_msg:
			{
				message::Process_manual_operation_msg t_Process_manual_operation_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Process_manual_operation_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Process_manual_operation_msg =  "<< t_Process_manual_operation_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eNotify_request_msg:
			{
				message::Notify_request_msg t_Notify_request_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Notify_request_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Notify_request_msg =  "<< t_Notify_request_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eNotify_response_msg:
			{
				message::Notify_response_msg t_Notify_response_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Notify_response_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Notify_response_msg =  "<< t_Notify_response_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eGround_detect_request_msg:
			{
				message::Ground_detect_request_msg t_Ground_detect_request_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Ground_detect_request_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Ground_detect_request_msg =  "<< t_Ground_detect_request_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eGround_detect_response_msg:
			{
				message::Ground_detect_response_msg t_Ground_detect_response_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Ground_detect_response_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Ground_detect_response_msg =  "<< t_Ground_detect_response_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eTerminal_status_msg:
			{
				message::Terminal_status_msg t_Terminal_status_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Terminal_status_msg.ParseFromString(p_msg->get_message_buf()))
				{
					//LOG(INFO) << " t_Terminal_status_msg =  "<< t_Terminal_status_msg.DebugString() << " --  " << this;
				}
				break;
			}
			case Communication_message::Message_type::eNotify_status_msg:
			{
				message::Notify_status_msg t_Notify_status_msg;
				//针对消息类型, 对消息进行二次解析
				if (t_Notify_status_msg.ParseFromString(p_msg->get_message_buf()))
				{
					LOG(INFO) << " t_Notify_status_msg =  "<< t_Notify_status_msg.DebugString() << " --  " << this;
				}
				break;
			}
			default :
				;
				break;
		}

	//无效的消息,
	return Error_manager(Error_code::INVALID_MESSAGE, Error_level::NEGLIGIBLE_ERROR,
						 " INVALID_MESSAGE error ");
}
//检查执行者的状态, 判断能否处理这条消息,
Error_manager System_executor::check_executer(Communication_message* p_msg)
{
	if ( p_msg == NULL )
	{
		return Error_manager(Error_code::POINTER_IS_NULL, Error_level::MINOR_ERROR,
							 "  POINTER IS NULL ");
	}

	//检查执行线程池
	Error_manager t_executor_result = System_executor::get_instance_references().check_status();
	if (t_executor_result.get_error_level() == NEGLIGIBLE_ERROR)//一级故障,轻微故障,
	{
		std::cout << "executer_is_busy , " << std::endl;
		//返回繁忙之后, 通信模块1秒后再次调用check
		return Error_code::COMMUNICATION_EXCUTER_IS_BUSY;
	}
	else if (t_executor_result.get_error_level() > NEGLIGIBLE_ERROR)
	{
		return Error_manager(Error_code::SYSTEM_EXECUTOR_STATUS_ERROR, Error_level::MINOR_ERROR,
							"System_executor::get_instance_references().check_status(); fun error ");
	}

	return Error_code::SUCCESS;
}


//处理消息的执行函数
Error_manager System_executor::execute_msg(Communication_message* p_msg)
{
	if ( p_msg == NULL )
	{
		return Error_manager(Error_code::POINTER_IS_NULL, Error_level::MINOR_ERROR,
							 "  POINTER IS NULL ");
	}



	return Error_code::SUCCESS;
}

//检查状态
Error_manager System_executor::check_status()
{
	if ( m_system_executor_status == SYSTEM_EXECUTOR_READY )
	{
		if ( m_thread_pool.thread_is_full_load() == false )
		{
		    return Error_code::SUCCESS;
		}
		else
		{
		    return Error_manager(Error_code::SYSTEM_EXECUTOR_STATUS_BUSY, Error_level::NEGLIGIBLE_ERROR,
		    					" System_executor::check_status error ");
		}
	}
	else
	{
		return Error_manager(Error_code::SYSTEM_EXECUTOR_STATUS_ERROR, Error_level::MINOR_ERROR,
							" System_executor::check_status error ");
	}
}

//定时发送状态信息
Error_manager System_executor::encapsulate_send_status()
{
	return Error_code::SUCCESS;
}

//定时发送 调度管理的状态
Error_manager System_executor::encapsulate_send_dispatch_manager_status()
{
return Error_code::SUCCESS;
}


//判断是否为待机，如果已经准备好，则可以执行任务。
bool System_executor::is_ready()
{
	if ( m_system_executor_status == SYSTEM_EXECUTOR_READY && m_thread_pool.thread_is_full_load() == false )
	{
	    return true;
	}
	else
	{
		return false;
	}
}

System_executor::System_executor_status System_executor::get_system_executor_status()
{
	return m_system_executor_status;
}


